"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.plugin = exports.KafkaNodePlugin = void 0;
const core_1 = require("@opentelemetry/core");
const api_1 = require("@opentelemetry/api");
const shimmer = __importStar(require("shimmer"));
const enums_1 = require("./enums");
const utils_1 = require("./utils");
const version_1 = require("./version");
class KafkaNodePlugin extends core_1.BasePlugin {
    constructor(moduleName) {
        super('@mzahor-test-org/open-telemetry-kafka-node', version_1.VERSION);
        this.moduleName = moduleName;
    }
    patch() {
        var _a, _b;
        this._logger.debug('kafka-node: patch kafka-node plugin');
        shimmer.wrap((_a = this._moduleExports.Producer) === null || _a === void 0 ? void 0 : _a.prototype, 'send', this._getProducerSendPatch.bind(this));
        shimmer.wrap((_b = this._moduleExports.Consumer) === null || _b === void 0 ? void 0 : _b.prototype, 'emit', this._getConsumerEmit.bind(this));
        return this._moduleExports;
    }
    unpatch() {
        var _a, _b;
        this._logger.debug('kafka-node: unpatch kafka-node plugin');
        shimmer.unwrap((_a = this._moduleExports.Producer) === null || _a === void 0 ? void 0 : _a.prototype, 'send');
        shimmer.unwrap((_b = this._moduleExports.Consumer) === null || _b === void 0 ? void 0 : _b.prototype, 'emit');
    }
    _getConsumerEmit(original) {
        const self = this;
        return function (event, ...args) {
            var _a;
            if (event !== 'message') {
                return original.apply(this, arguments);
            }
            const message = args[0];
            const brokers = Object.keys(this.client.brokers).join(',');
            const span = exports.plugin._tracer.startSpan(message.topic, {
                kind: api_1.SpanKind.CONSUMER,
                attributes: {
                    [enums_1.AttributeNames.MESSAGING_SYSTEM]: 'kafka',
                    [enums_1.AttributeNames.MESSAGING_DESTINATION]: message.topic,
                    [enums_1.AttributeNames.MESSAGING_DESTINATIONKIND]: 'topic',
                    [enums_1.AttributeNames.NET_PEER_NAME]: brokers
                },
            });
            if (message.offset)
                span.setAttribute(enums_1.AttributeNames.MESSAGING_OFFSET, message.offset);
            if (message.partition)
                span.setAttribute(enums_1.AttributeNames.MESSAGING_PARTITION, message.partition);
            if ((_a = self._config) === null || _a === void 0 ? void 0 : _a.consumerHook) {
                self._safeExecute([], () => self._config.consumerHook(span, message), false);
            }
            try {
                return exports.plugin._tracer.withSpan(span, () => {
                    return original.apply(this, arguments);
                });
            }
            finally {
                span.setStatus({
                    code: api_1.CanonicalCode.OK,
                    message: 'OK'
                });
                span.end();
            }
        };
    }
    _getProducerSendPatch(original) {
        const self = this;
        return function (payloads, origCb) {
            const spans = payloads.flatMap((payload) => {
                const brokers = Object.keys(this.client.brokers).join(',');
                let spansInThisRequest;
                if (Array.isArray(payload.messages)) {
                    spansInThisRequest = payload.messages.map(message => self._startProducerSpan(payload, message, brokers));
                }
                else {
                    spansInThisRequest = [self._startProducerSpan(payload, payload.messages, brokers)];
                }
                return spansInThisRequest;
            });
            const patchedCb = (error, data) => {
                const status = utils_1.parseResponseStatus(error, data);
                self._safeExecute(spans, () => origCb(error, data), true);
                spans.forEach(span => {
                    span.setStatus(status);
                    span.end();
                });
            };
            return original.call(this, payloads, patchedCb);
        };
    }
    _startProducerSpan(producerRequest, message, brokers) {
        var _a;
        const span = exports.plugin._tracer.startSpan(producerRequest.topic, {
            kind: api_1.SpanKind.PRODUCER,
            attributes: {
                [enums_1.AttributeNames.MESSAGING_SYSTEM]: 'kafka',
                [enums_1.AttributeNames.MESSAGING_DESTINATION]: producerRequest.topic,
                [enums_1.AttributeNames.MESSAGING_DESTINATIONKIND]: 'topic',
                [enums_1.AttributeNames.NET_PEER_NAME]: brokers
            },
        });
        if ((_a = this._config) === null || _a === void 0 ? void 0 : _a.producerHook) {
            this._safeExecute([], () => this._config.producerHook(span, producerRequest, message), false);
        }
        return span;
    }
    _safeExecute(spans, execute, rethrow) {
        try {
            return execute();
        }
        catch (error) {
            if (rethrow) {
                spans.forEach(span => {
                    span.setStatus({
                        code: api_1.CanonicalCode.INTERNAL,
                        message: error === null || error === void 0 ? void 0 : error.message
                    });
                    span.end();
                });
                throw error;
            }
            this._logger.error('caught error ', error);
        }
    }
}
exports.KafkaNodePlugin = KafkaNodePlugin;
exports.plugin = new KafkaNodePlugin('kafka-node');
//# sourceMappingURL=kafka-node.js.map