"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const src_1 = require("../src");
const core_1 = require("@opentelemetry/core");
const tracing_1 = require("@opentelemetry/tracing");
const node_1 = require("@opentelemetry/node");
const context_async_hooks_1 = require("@opentelemetry/context-async-hooks");
const api_1 = require("@opentelemetry/api");
const kafkaNode = __importStar(require("kafka-node"));
const kafka_node_1 = require("kafka-node");
const enums_1 = require("../src/enums");
describe('aspecto-open-telemetry-kafka-node', () => {
    const logger = new core_1.NoopLogger();
    const provider = new node_1.NodeTracerProvider();
    const memoryExporter = new tracing_1.InMemorySpanExporter();
    const spanProcessor = new tracing_1.SimpleSpanProcessor(memoryExporter);
    provider.addSpanProcessor(spanProcessor);
    let contextManager;
    let kafkaClient;
    const getKafkaProducerSpans = () => {
        return memoryExporter
            .getFinishedSpans()
            .filter((s) => s.kind === api_1.SpanKind.PRODUCER);
    };
    const getKafkaConsumerSpans = () => {
        return memoryExporter
            .getFinishedSpans()
            .filter((s) => s.kind === api_1.SpanKind.CONSUMER);
    };
    beforeEach(() => {
        contextManager = new context_async_hooks_1.AsyncHooksContextManager();
        api_1.context.setGlobalContextManager(contextManager.enable());
        this.kafkaClient = new kafka_node_1.KafkaClient({
            kafkaHost: 'testing_mock_host:1234'
        });
    });
    afterEach(() => {
        memoryExporter.reset();
        contextManager.disable();
        this.kafkaClient.close();
        src_1.plugin.disable();
    });
    describe('successful producer.send', () => {
        beforeEach(() => {
            kafkaNode.Producer.prototype.send = function (payloads, cb) {
                try {
                    cb(undefined, {});
                }
                catch (err) { }
            };
            src_1.plugin.enable(kafkaNode, provider, logger);
        });
        it('producer.send create span with right attributes', (done) => {
            const producer = new kafka_node_1.Producer(this.kafkaClient);
            const producerRequest = {
                topic: 'topic-name',
                messages: 'message content'
            };
            producer.send([producerRequest], (error, data) => {
                setImmediate(() => {
                    const spans = getKafkaProducerSpans();
                    expect(spans.length).toBe(1);
                    const span = spans[0];
                    expect(span.name).toStrictEqual('topic-name');
                    expect(span.status.code).toStrictEqual(api_1.CanonicalCode.OK);
                    expect(span.status.message).toStrictEqual('OK');
                    expect(span.attributes[enums_1.AttributeNames.MESSAGING_SYSTEM]).toStrictEqual('kafka');
                    expect(span.attributes[enums_1.AttributeNames.MESSAGING_DESTINATION]).toStrictEqual('topic-name');
                    expect(span.attributes[enums_1.AttributeNames.MESSAGING_DESTINATIONKIND]).toStrictEqual('topic');
                    expect(span.attributes[enums_1.AttributeNames.NET_PEER_NAME]).toStrictEqual('testing_mock_host:1234');
                    done();
                });
            });
        });
        it('two messages in request should create two spans', (done) => {
            const producer = new kafka_node_1.Producer(this.kafkaClient);
            const producerRequest = {
                topic: 'topic-name',
                messages: ['message content 1', 'message content 2']
            };
            producer.send([producerRequest], (error, data) => {
                setImmediate(() => {
                    const spans = getKafkaProducerSpans();
                    expect(spans.length).toBe(2);
                    spans.forEach(span => {
                        expect(span.name).toStrictEqual('topic-name');
                        expect(span.attributes[enums_1.AttributeNames.MESSAGING_SYSTEM]).toStrictEqual('kafka');
                        expect(span.attributes[enums_1.AttributeNames.MESSAGING_DESTINATION]).toStrictEqual('topic-name');
                        expect(span.attributes[enums_1.AttributeNames.MESSAGING_DESTINATIONKIND]).toStrictEqual('topic');
                    });
                    done();
                });
            });
        });
        it('two requests in same send should create two spans', (done) => {
            const producer = new kafka_node_1.Producer(this.kafkaClient);
            const req1 = {
                topic: 'topic-name-1',
                messages: 'message content 1'
            };
            const req2 = {
                topic: 'topic-name-1',
                messages: 'message content 1'
            };
            producer.send([req1, req2], (error, data) => {
                setImmediate(() => {
                    const spans = getKafkaProducerSpans();
                    expect(spans.length).toBe(2);
                    const [s1, s2] = spans;
                    expect(s1.name).toStrictEqual(req1.topic);
                    expect(s1.attributes[enums_1.AttributeNames.MESSAGING_DESTINATION]).toStrictEqual(req1.topic);
                    expect(s2.name).toStrictEqual(req2.topic);
                    expect(s2.attributes[enums_1.AttributeNames.MESSAGING_DESTINATION]).toStrictEqual(req2.topic);
                    done();
                });
            });
        });
        it('multiple requests and messages', (done) => {
            const producer = new kafka_node_1.Producer(this.kafkaClient);
            const req1 = {
                topic: 'topic-name-1',
                messages: ['message content 1', 'message content 2', 'message content 3']
            };
            const req2 = {
                topic: 'topic-name-1',
                messages: ['message content 1', 'message content 2']
            };
            producer.send([req1, req2], (error, data) => {
                setImmediate(() => {
                    const spans = getKafkaProducerSpans();
                    expect(spans.length).toBe(5);
                    done();
                });
            });
        });
        it('send cb throw exception', (done) => {
            const producer = new kafka_node_1.Producer(this.kafkaClient);
            const req1 = {
                topic: 'topic-name-1',
                messages: 'message content 1'
            };
            producer.send([req1], (error, data) => {
                setImmediate(() => {
                    const spans = getKafkaProducerSpans();
                    expect(spans.length).toBe(1);
                    done();
                });
                throw new Error('exception thrown in send callback');
            });
        });
    });
    describe('failure producer.send', () => {
        beforeEach(() => {
            kafkaNode.Producer.prototype.send = function (payloads, cb) {
                setImmediate(() => cb(new Error('error message from cb'), undefined));
            };
            src_1.plugin.enable(kafkaNode, provider, logger);
        });
        it('error returned in cb sets the span status', done => {
            const producer = new kafka_node_1.Producer(this.kafkaClient);
            const req1 = {
                topic: 'topic-name-1',
                messages: 'message content 1'
            };
            producer.send([req1], (error, data) => {
                setImmediate(() => {
                    const spans = getKafkaProducerSpans();
                    expect(spans.length).toBe(1);
                    expect(spans[0].status.code).toBe(api_1.CanonicalCode.UNKNOWN);
                    expect(spans[0].status.message).toStrictEqual('error message from cb');
                    done();
                });
            });
        });
    });
    describe('producer hook add attribute', () => {
        beforeEach(() => {
            kafkaNode.Producer.prototype.send = function (payloads, cb) {
                setImmediate(() => cb(undefined, {}));
            };
            const config = {
                enabled: true,
                producerHook: (span, producerRequest, message) => {
                    span.setAttribute('attribute from testing hook', message);
                }
            };
            src_1.plugin.enable(kafkaNode, provider, logger, config);
        });
        it('custom attributes from hook are added to span', done => {
            const producer = new kafka_node_1.Producer(this.kafkaClient);
            const req1 = {
                topic: 'topic-name-1',
                messages: ['message content 1', 'message content 2']
            };
            producer.send([req1], (error, data) => {
                setImmediate(() => {
                    const spans = getKafkaProducerSpans();
                    expect(spans.length).toBe(2);
                    const [s1, s2] = spans;
                    expect(s1.attributes['attribute from testing hook']).toStrictEqual('message content 1');
                    expect(s2.attributes['attribute from testing hook']).toStrictEqual('message content 2');
                    done();
                });
            });
        });
    });
    describe('producer hook throws exception', () => {
        beforeEach(() => {
            kafkaNode.Producer.prototype.send = function (payloads, cb) {
                setImmediate(() => cb(undefined, {}));
            };
            const config = {
                enabled: true,
                producerHook: (span, producerRequest, message) => {
                    throw Error('exception thrown from producer hook');
                }
            };
            src_1.plugin.enable(kafkaNode, provider, logger, config);
        });
        it('spans should be created when exception thrown in hook', done => {
            const producer = new kafka_node_1.Producer(this.kafkaClient);
            const req1 = {
                topic: 'topic-name-1',
                messages: ['message content 1', 'message content 2']
            };
            producer.send([req1], (error, data) => {
                setImmediate(() => {
                    const spans = getKafkaProducerSpans();
                    expect(spans.length).toBe(2);
                    expect(spans[0].status.code).toBe(api_1.CanonicalCode.OK);
                    done();
                });
            });
        });
    });
    describe('kafka consumer', () => {
        beforeEach(() => {
            src_1.plugin.enable(kafkaNode, provider, logger);
        });
        it('consumer create span with right attributes', () => {
            const consumer = new kafka_node_1.Consumer(this.kafkaClient, [], {});
            const msg = {
                topic: 'topic-name',
                value: 'content of message',
                offset: 1234,
                partition: 4
            };
            consumer.emit('message', msg);
            const spans = getKafkaConsumerSpans();
            expect(spans.length).toBe(1);
            const span = spans[0];
            expect(span.name).toStrictEqual(msg.topic);
            expect(span.status.code).toStrictEqual(api_1.CanonicalCode.OK);
            expect(span.status.message).toStrictEqual('OK');
            expect(span.attributes[enums_1.AttributeNames.MESSAGING_SYSTEM]).toStrictEqual('kafka');
            expect(span.attributes[enums_1.AttributeNames.MESSAGING_DESTINATION]).toStrictEqual(msg.topic);
            expect(span.attributes[enums_1.AttributeNames.MESSAGING_DESTINATIONKIND]).toStrictEqual('topic');
            expect(span.attributes[enums_1.AttributeNames.MESSAGING_PARTITION]).toStrictEqual(msg.partition);
            expect(span.attributes[enums_1.AttributeNames.MESSAGING_OFFSET]).toStrictEqual(msg.offset);
            expect(span.attributes[enums_1.AttributeNames.NET_PEER_NAME]).toStrictEqual('testing_mock_host:1234');
        });
        it('data missing in message should not create attribute', () => {
            const consumer = new kafka_node_1.Consumer(this.kafkaClient, [], {});
            const msg = {
                topic: 'topic-name',
                value: 'content of message',
            };
            consumer.emit('message', msg);
            const spans = getKafkaConsumerSpans();
            expect(spans.length).toBe(1);
            const span = spans[0];
            expect(span.attributes.hasOwnProperty(enums_1.AttributeNames.MESSAGING_PARTITION)).toBeFalsy();
            expect(span.attributes.hasOwnProperty(enums_1.AttributeNames.MESSAGING_OFFSET)).toBeFalsy();
        });
        it('ignoring events not of type `message`', () => {
            const consumer = new kafka_node_1.Consumer(this.kafkaClient, [], {});
            consumer.emit('some other event', new Error('some error which we dont care about'));
            const spans = getKafkaConsumerSpans();
            expect(spans.length).toBe(0);
        });
    });
    describe('kafka consumer custom hook', () => {
        beforeEach(() => {
            const config = {
                enabled: true,
                consumerHook: (span, message) => {
                    span.setAttribute('attribute from testing hook', message.value);
                }
            };
            src_1.plugin.enable(kafkaNode, provider, logger, config);
        });
        it('attribute from consumer hook should be set on span', () => {
            const consumer = new kafka_node_1.Consumer(this.kafkaClient, [], {});
            const msg = {
                topic: 'topic-name',
                value: 'content of message',
            };
            consumer.emit('message', msg);
            const spans = getKafkaConsumerSpans();
            expect(spans.length).toBe(1);
            const span = spans[0];
            expect(span.attributes['attribute from testing hook']).toStrictEqual(msg.value);
        });
    });
    describe('kafka consumer custom hook throws', () => {
        beforeEach(() => {
            const config = {
                enabled: true,
                consumerHook: (span, message) => {
                    throw Error('exception thrown from consumer hook');
                }
            };
            src_1.plugin.enable(kafkaNode, provider, logger, config);
        });
        it('spans should be created when exception thrown in hook', () => {
            const consumer = new kafka_node_1.Consumer(this.kafkaClient, [], {});
            const msg = {
                topic: 'topic-name',
                value: 'content of message',
            };
            consumer.emit('message', msg);
            const spans = getKafkaConsumerSpans();
            expect(spans.length).toBe(1);
            const span = spans[0];
            expect(span.status.code).toStrictEqual(api_1.CanonicalCode.OK);
        });
    });
});
//# sourceMappingURL=aspecto-open-telemetry-kafka-node.spec.js.map